@@ -635,7 +635,7 @@ text_entry_callback (GtkWidget *w, gpointer data)
   GSGDialogElement *elem = data;
   const SANE_Option_Descriptor *opt;
   GSGDialog *dialog = elem->dialog;
-  gchar *text;
+  const gchar *text;
   int opt_num;
   char *buf;
 
@@ -829,7 +829,11 @@ panel_destroy (GSGDialog * dialog)
   /* pre 0.99.4: */
   gtk_tooltips_unref (dialog->tooltips);
 #else
+# if GTK_MAJOR_VERSION == 2
+  gtk_object_sink (GTK_OBJECT (dialog->tooltips));
+# else
   gtk_object_unref (GTK_OBJECT (dialog->tooltips));
+# endif /* GTK_MAJOR_VERSION == 2 */
 #endif
   gtk_widget_destroy (dialog->main_hbox);
 
@@ -900,8 +904,13 @@ panel_build (GSGDialog * dialog)
   dialog->tooltips_bg.green = 59113;
   dialog->tooltips_bg.blue  = 35979;
   gdk_color_alloc (gtk_widget_get_colormap (main_hbox), &dialog->tooltips_bg);
+
+  /* GTK2 doesn't have the function (seems like it's not needed with GTK2 anyway) */
+#if GTK_MAJOR_VERSION < 2
   gtk_tooltips_set_colors (dialog->tooltips,
 			   &dialog->tooltips_bg, &dialog->tooltips_fg);
+#endif /* GTK_MAJOR_VERSION < 2 */
+
   gsg_set_tooltips (dialog, preferences.tooltips_enabled);
 
   gtk_container_add (GTK_CONTAINER (dialog->window), main_hbox);
