@@ -463,15 +463,16 @@ restore_option (Preview *p, int option, SANE_Word saved_value, int valid)
     }
 }
 
-static void
+static SANE_Status
 set_option_float (Preview *p, int option, float value)
 {
   const SANE_Option_Descriptor *opt;
   SANE_Handle dev;
   SANE_Word word;
+  SANE_Status status;
 
   if (option <= 0 || value <= -INF || value >= INF)
-    return;
+    return SANE_STATUS_INVAL;
 
   dev = p->dialog->dev;
   opt = sane_get_option_descriptor (dev, option);
@@ -479,7 +480,17 @@ set_option_float (Preview *p, int option, float value)
     word = SANE_FIX (value) + 0.5;
   else
     word = value + 0.5;
-  sane_control_option (dev, option, SANE_ACTION_SET_VALUE, &word, 0);
+  status = sane_control_option (dev, option, SANE_ACTION_SET_VALUE, &word, 0);
+  if (status != SANE_STATUS_GOOD)
+    {
+      char buf[256];
+      opt = sane_get_option_descriptor (dev, option);
+      snprintf (buf, sizeof (buf), "Failed to set option %s: %s.",
+		opt->name, sane_strstatus (status));
+      gsg_error (buf);
+      return status;
+    }
+  return SANE_STATUS_GOOD;
 }
 
 static void
@@ -1206,12 +1217,20 @@ preview_new (GSGDialog *dialog)
   /* Connect the motion-notify events of the preview area with the
      rulers.  Nifty stuff!  */
 
+#if GTK_MAJOR_VERSION == 2
+  class = GTK_WIDGET_CLASS (GTK_OBJECT_GET_CLASS (GTK_OBJECT (p->hruler)));
+#else
   class = GTK_WIDGET_CLASS (GTK_OBJECT (p->hruler)->klass);
+#endif /* GTK_MAJOR_VERSION == 2 */
   signal_func = (GtkSignalFunc) class->motion_notify_event;
   gtk_signal_connect_object (GTK_OBJECT (p->window), "motion_notify_event",
 			     signal_func, GTK_OBJECT (p->hruler));
 
+#if GTK_MAJOR_VERSION == 2
+  class = GTK_WIDGET_CLASS (GTK_OBJECT_GET_CLASS (GTK_OBJECT (p->vruler)));
+#else
   class = GTK_WIDGET_CLASS (GTK_OBJECT (p->vruler)->klass);
+#endif /* GTK_MAJOR_VERSION == 2 */
   signal_func = (GtkSignalFunc) class->motion_notify_event;
   gtk_signal_connect_object (GTK_OBJECT (p->window), "motion_notify_event",
 			     signal_func, GTK_OBJECT (p->vruler));
@@ -1382,6 +1401,7 @@ preview_scan (Preview *p)
   const SANE_Option_Descriptor *opt;
   gint gwidth, gheight;
   int i;
+  SANE_Status status;
 
   save_option (p, p->dialog->well_known.dpi,
 	       &p->saved_dpi, &p->saved_dpi_valid);
@@ -1423,8 +1443,9 @@ preview_scan (Preview *p)
 	dpi = min;
       if (dpi > max)
 	dpi = max;
-
-      set_option_float (p, p->dialog->well_known.dpi, dpi);
+      status = set_option_float (p, p->dialog->well_known.dpi, dpi);
+      if (status != SANE_STATUS_GOOD)
+	return;
     }
 
   /* set the scan window (necessary since backends may default to
