@@ -896,7 +896,7 @@ int
 main (int argc, char **argv)
 {
   GtkWidget *menu, *menu_bar, *menu_bar_item, *preview_vbox;
-  GtkWidget *hbox, *vbox, *button, *alignment, *frame;
+  GtkWidget *hbox, *vbox, *button, *alignment, *frame, *scrolled_window;
   int i;
 
   prog_name = strrchr (argv[0], '/');
@@ -1025,7 +1025,18 @@ main (int argc, char **argv)
   }
 #endif
 
-  gtk_container_add (GTK_CONTAINER (hbox), win.dialog_box);
+  /* use a scrolled window to show the device options, as in xscanimage */
+  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
+				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  gtk_scrolled_window_set_placement (GTK_SCROLLED_WINDOW (scrolled_window),
+				     GTK_CORNER_TOP_RIGHT);
+  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled_window),
+					 win.dialog_box);
+  gtk_container_add (GTK_CONTAINER (hbox), scrolled_window);
+
+  gtk_widget_show (GTK_WIDGET (scrolled_window));
+
   if (device[0])
     {
       switch_device (device[0]);
