@@ -64,10 +64,14 @@
 
 # ifdef HAVE_LIBGIMP_GIMPFEATURES_H
 #  include <libgimp/gimpfeatures.h>
-# else
+# elif !defined(ENABLE_GIMP_1_3)
 #  define GIMP_CHECK_VERSION(major, minor, micro) 0
 # endif /* HAVE_LIBGIMP_GIMPFEATURES_H */
 
+#ifdef ENABLE_GIMP_1_3
+# define GIMP_HAVE_RESOLUTION_INFO
+#endif /* ENABLE_GIMP_1_3 */
+
 # ifdef GIMP_CHECK_VERSION
 #  if GIMP_CHECK_VERSION(1,1,25)
 /* ok, we have the new gimp interface */
@@ -76,8 +80,10 @@
 #   include "xscanimage-gimp-1_0-compat.h" 
 #  endif
 # else
+#  ifndef ENABLE_GIMP_1_3
 /* we have the old gimp interface and need the compatibility header file */
-#  include "xscanimage-gimp-1_0-compat.h" 
+#   include "xscanimage-gimp-1_0-compat.h" 
+#  endif /* !ENABLE_GIMP_1_3 */
 # endif
 
 static void query (void);
@@ -299,7 +305,7 @@ query (void)
       "Andy Beck, Tristan Tarrant, and David Mosberger",
       "Andy Beck, Tristan Tarrant, and David Mosberger",
       "8th June 1997",
-      "<Toolbox>/Xtns/Acquire Image/Device dialog...",
+      "<Toolbox>/File/Acquire/xscanimage/Device dialog...",
       "RGB, GRAY",
       GIMP_EXTENSION,
       nargs, nreturn_vals,
@@ -314,7 +320,7 @@ query (void)
       if (encode_devname (devlist[i]->name, sizeof (name) - 11, name + 11) < 0)
 	continue;	/* name too long... */
 
-      strncpy (mpath, "<Toolbox>/Xtns/Acquire Image/", sizeof (mpath));
+      strncpy (mpath, "<Toolbox>/File/Acquire/xscanimage/", sizeof (mpath));
       len = strlen (mpath);
       for (j = 0; devlist[i]->name[j]; ++j)
 	{
@@ -343,7 +349,11 @@ run (char *name, int nparams, GimpParam * param,
      int *nreturn_vals, GimpParam ** return_vals)
 {
   static GimpParam values[2];
+#ifdef ENABLE_GIMP_1_3
+  GimpRunMode run_mode;
+#else
   GimpRunModeType run_mode;
+#endif /* ENABLE_GIMP_1_3 */
   char devname[1024];
   char *args[2];
   int nargs;
@@ -394,6 +404,43 @@ null_print_func (const gchar *msg)
 
 #endif /* HAVE_LIBGIMP_GIMP_H */
 
+static 
+SANE_Word get_resolution (SANE_Handle dev)
+{
+  SANE_Status status;
+  SANE_Word resolution;
+  SANE_Int num_options, i;
+  const SANE_Option_Descriptor *option_desc;
+
+  status = sane_control_option (dev, 0, SANE_ACTION_GET_VALUE, &num_options, 0);
+  if (status != SANE_STATUS_GOOD)
+    return 0;
+  
+  for (i = 1; i < num_options; i++)
+    {
+      option_desc = sane_get_option_descriptor (dev, i);
+      if (option_desc)
+	if (option_desc->name)	    
+	  {
+	    if (strncmp (option_desc->name, SANE_NAME_SCAN_RESOLUTION, 
+			 sizeof(SANE_NAME_SCAN_RESOLUTION)) == 0)
+	      {
+		sane_control_option (dev, i, SANE_ACTION_GET_VALUE,
+				     &resolution, 0);
+		if (status == SANE_STATUS_GOOD) 
+		  {
+		    if (option_desc->type == SANE_TYPE_INT)
+		      return resolution;
+		    else if (option_desc->type == SANE_TYPE_FIXED)
+		      return (SANE_Word) SANE_UNFIX (resolution);
+		  }
+		return 0;		  
+	      }
+	  }
+    }      
+  return 0; 
+}
+
 static void
 update_preview (GSGDialog *dialog, void *arg)
 {
@@ -923,6 +970,11 @@ scan_start (void)
 {
   SANE_Status status;
   SANE_Handle dev = gsg_dialog_get_device (dialog);
+
+#ifdef HAVE_LIBGIMP_GIMP_H  
+  SANE_Word resolution;
+#endif /* HAVE_LIBGIMP_GIMP_H */
+  
   const char *frame_type = 0;
   char buf[256];
   int fd;
@@ -958,6 +1010,10 @@ scan_start (void)
     }
 #endif /* HAVE_LIBGIMP_GIMP_H */
 
+#ifdef GIMP_HAVE_RESOLUTION_INFO	  
+  resolution = get_resolution(dev);
+#endif
+
   scan_win.x = scan_win.y = 0;
 
   while (gtk_events_pending ())
@@ -1087,6 +1143,14 @@ scan_start (void)
 
 	  scan_win.image_ID = gimp_image_new (scan_win.param.pixels_per_line,
 					     scan_win.param.lines, image_type);
+
+/* the following is supported since gimp-1.1.0 */
+#ifdef GIMP_HAVE_RESOLUTION_INFO	  
+	  if (resolution > 0)
+	    gimp_image_set_resolution(scan_win.image_ID, resolution, 
+				      resolution);
+#endif	  
+
 	  layer_ID = gimp_layer_new (scan_win.image_ID, "Background",
 				     scan_win.param.pixels_per_line,
 				     scan_win.param.lines,
@@ -1234,7 +1298,8 @@ static void
 update_double_callback (GtkWidget *widget, gpointer data)
 {
   double *valuep = data;
-  char *start, *end;
+  const char *start;
+  char *end;
   double v;
 
   start = gtk_entry_get_text (GTK_ENTRY (widget));
@@ -1793,7 +1858,13 @@ init (int argc, char **argv)
 #ifdef HAVE_LIBGIMP_GIMP_H
   gtk_rc_parse (gimp_gtkrc ());
 
+# ifdef ENABLE_GIMP_1_3
+  /* GIMP 1.3 defines gimp_use_xshm() as a macro always returning TRUE
+   * (in the compat header) */
+  gdk_set_use_xshm (TRUE);
+# else
   gdk_set_use_xshm (gimp_use_xshm ());
+# endif /* ENABLE_GIMP_1_3 */
 #endif
 
   gsg_make_path (sizeof (filename), filename, 0, "sane-style", 0, ".rc");
@@ -1952,17 +2023,17 @@ main (int argc, char **argv)
     old_print_func = g_set_print_handler (null_print_func);
     old_printerr_func = g_set_printerr_handler(null_print_func);
     /* gimp_main () returns 1 if xscanimage wasn't invoked by GIMP */
-#ifdef HAVE_OS2_H
+# ifdef HAVE_OS2_H
     set_gimp_PLUG_IN_INFO(&PLUG_IN_INFO);
-#endif
+# endif
     result = gimp_main (argc, argv);
-    g_set_message_handler (old_print_func);
-    g_set_error_handler(old_printerr_func);
+    g_set_print_handler (old_print_func);
+    g_set_printerr_handler (old_printerr_func);
     if (result)
       interface (argc, argv);
   }
 #else
   interface (argc, argv);
-#endif
+#endif /* HAVE_LIBGIMP_GIMP_H */
   return 0;
 }
